@@ -2,10 +2,12 @@ use std::collections::HashMap;
 use std::fs::File;
 use std::io::{BufRead, BufReader};
 use bevy::prelude::*;
+use plugin::{ButtonState, GergButton, GergLabel, GergPictureBox};
 
 use crate::colors::parse_color;
 
 mod colors;
+pub mod plugin;
 
 #[derive(Default)]
 struct GlobalSettings {
@@ -57,7 +59,8 @@ impl Fields {
 
 enum ControlType {
     PictureBox,
-    Label
+    Label,
+    Button
 }
 impl Default for ControlType {
     fn default() -> Self { ControlType::PictureBox }
@@ -69,6 +72,13 @@ enum ReadState {
     Control
 }
 
+pub fn instantiate_controls_from_file(filename: &str) -> Controls {
+    let lines = read_ui_file(filename);
+    let controls = instantiate_controls(lines);
+
+    return controls;
+}
+
 pub fn read_ui_file(ui_filename: &str) -> Vec<String> {
 
     let filename = format!("assets/{}", ui_filename);
@@ -85,13 +95,6 @@ pub fn read_ui_file(ui_filename: &str) -> Vec<String> {
     return list;
 }
 
-pub fn instantiate_controls_from_file(filename: &str) -> Controls {
-    let lines = read_ui_file(filename);
-    let controls = instantiate_controls(lines);
-
-    return controls;
-}
-
 pub fn instantiate_controls(lines: Vec<String>) -> Controls {
 
     let mut controls = HashMap::new();
@@ -109,23 +112,56 @@ pub fn instantiate_controls(lines: Vec<String>) -> Controls {
         if line.is_empty() { continue; } // skip line
         if line.trim().starts_with("//") { continue; } // skip line
 
+        if line.trim().starts_with("--") {
+            match line.to_lowercase().as_str() {
+                "--global_settings--" => { },
+                "--picture_box--" => { },
+                "--label--" => { },
+                "--button--" => { },
+                "--end--" => { },
+                _ => {
+                    panic!("Control type [{}] unknown.", line);
+                }
+            }
+        }
+
         match line.to_lowercase().as_str() {
             "--global_settings--" => {
                 read_state = ReadState::GlobalSettings;
                 global_settings = GlobalSettings { ..Default::default() };
+                global_settings.font_size = "20".to_string();
+                global_settings.color = "255;255;255".to_string();
             },
             "--picture_box--" => {
                 read_state = ReadState::Control;
                 control.control_type = ControlType::PictureBox;
-
+                control.fields.insert("draw_order".to_string(), "0.0".to_string());
+                control.fields.insert("top_left_position".to_string(), "".to_string());
+                control.fields.insert("center_position".to_string(), "".to_string());
+                control.fields.insert("color".to_string(), global_settings.color.clone());
             },
             "--label--" => {
                 read_state = ReadState::Control;
                 control.control_type = ControlType::Label;
+                control.fields.insert("top_left_position".to_string(), "".to_string());
+                control.fields.insert("center_position".to_string(), "".to_string());
+                control.fields.insert("color".to_string(), global_settings.color.clone());
                 control.fields.insert("font_name".to_string(), global_settings.font_name.clone());
                 control.fields.insert("font_size".to_string(), global_settings.font_size.clone());
-                control.fields.insert("color".to_string(), global_settings.color.clone());
+                control.fields.insert("text_string".to_string(), "".to_string());
             },
+            "--button--" => {
+                read_state = ReadState::Control;
+                control.control_type = ControlType::Button;
+                control.fields.insert("texture_name_hover".to_string(), "".to_string());
+                control.fields.insert("texture_name_active".to_string(), "".to_string());
+                control.fields.insert("texture_name_disabled".to_string(), "".to_string());
+                control.fields.insert("on_click_sound".to_string(), "".to_string());
+                control.fields.insert("draw_order".to_string(), "0.0".to_string());
+                control.fields.insert("top_left_position".to_string(), "".to_string());
+                control.fields.insert("center_position".to_string(), "".to_string());
+                control.fields.insert("color".to_string(), global_settings.color.clone());
+            }
             "--end--" => {
                 match read_state {
                     ReadState::None => { panic!("End found while not in a valid state. Line #{}: {}.", line_number, line); },
@@ -183,33 +219,73 @@ pub fn spawn_controls(commands: &mut Commands, asset_server: Res<AssetServer>, m
 
         match control.control_type {
             ControlType::PictureBox => {
+                let center_position = Vec3::new(top_left_position.x + size.x * 0.5, top_left_position.y - size.y * 0.5, parse_f32(control.fields.get_by_name("draw_order")));
                 let scale = Vec3::new(1.0, 1.0, 1.0);
-                let material_path = control.fields.get_by_name("texture_name");
-                let texture_handle = asset_server.load(material_path.as_str());
-                let material = materials.add(texture_handle.into());
-            
-                let draw_order = control.fields.get_by_name("draw_order");
-                let center_position = Vec3::new(top_left_position.x + size.x * 0.5, top_left_position.y - size.y * 0.5, parse_f32(draw_order));
+                let texture_name = control.fields.get_by_name("texture_name");
+                let color_material_handle = get_color_material_handle(texture_name, &asset_server, control, &mut materials);
 
-                let bundle = instantiate_sprite_bundle(size, center_position, scale, material);
-                let entity = commands.spawn_bundle(bundle).id();
+                let bundle = instantiate_sprite_bundle(size, center_position, scale, color_material_handle, true);
+                let entity = commands
+                    .spawn_bundle(bundle)
+                    .insert(GergPictureBox)
+                    .id();
 
                 results.push(entity);
             },
             ControlType::Label => {
-                let min_size = Vec2::new(0.0, 0.0);
-                let font_path = format!("fonts/{}", control.fields.get_by_name("font_name"));
-                let font_handle: Handle<Font> = asset_server.load(font_path.as_str());
-            
                 top_left_position.x += screen_size.x * 0.5;
                 top_left_position.y = screen_size.y * 0.5 - top_left_position.y;
+                let min_size = Vec2::new(0.0, 0.0);
                 let text = control.fields.get_by_name("text_string");
+                let font_handle: Handle<Font> = asset_server.load(format!("fonts/{}", control.fields.get_by_name("font_name")).as_str());
                 let font_size = parse_f32(control.fields.get_by_name("font_size"));
-                let color_u32 = parse_color(control.fields.get_by_name("color"));
-                let color = to_bevy_color(color_u32);
+                let color = to_bevy_color(parse_color(control.fields.get_by_name("color")));
 
                 let bundle = instantiate_textbundle(top_left_position, min_size, size, text, font_handle, font_size, color);
-                let entity = commands.spawn_bundle(bundle).id();
+                let entity = commands
+                    .spawn_bundle(bundle)
+                    .insert(GergLabel)
+                    .id();
+
+                results.push(entity);
+            },
+            ControlType::Button => {
+                let center_position = Vec3::new(top_left_position.x + size.x * 0.5, top_left_position.y - size.y * 0.5, parse_f32(control.fields.get_by_name("draw_order")));
+                let scale = Vec3::new(1.0, 1.0, 1.0);
+
+                let texture_name_normal = control.fields.get_by_name("texture_name_normal");
+                let mut texture_name_hover = control.fields.get_by_name("texture_name_hover");
+                if texture_name_hover.is_empty() {
+                    texture_name_hover = texture_name_normal;
+                }
+                let mut texture_name_active = control.fields.get_by_name("texture_name_active");
+                if texture_name_active.is_empty() {
+                    texture_name_active = texture_name_normal;
+                }
+                let mut texture_name_disabled = control.fields.get_by_name("texture_name_disabled");
+                if texture_name_disabled.is_empty() {
+                    texture_name_disabled = texture_name_normal;
+                }
+
+                let color_material_handle_normal = get_color_material_handle(texture_name_normal, &asset_server, control, &mut materials);
+                let color_material_handle_hover = get_color_material_handle(texture_name_hover, &asset_server, control, &mut materials);
+                let color_material_handle_active = get_color_material_handle(texture_name_active, &asset_server, control, &mut materials);
+                let color_material_handle_disabled = get_color_material_handle(texture_name_disabled, &asset_server, control, &mut materials);
+                
+                let on_click_sound = control.fields.get_by_name("on_click_sound");
+
+                let bundle = instantiate_sprite_bundle(size, center_position, scale, color_material_handle_normal.clone(), true);
+                let entity = commands
+                    .spawn_bundle(bundle)
+                    .insert(GergButton {
+                        button_state: ButtonState::Normal,
+                        color_material_handle_normal: color_material_handle_normal,
+                        color_material_handle_hover: color_material_handle_hover,
+                        color_material_handle_active: color_material_handle_active,
+                        color_material_handle_disabled: color_material_handle_disabled,
+                        on_click_sound: on_click_sound.to_string()
+                    })
+                    .id();
 
                 results.push(entity);
             }
@@ -221,13 +297,25 @@ pub fn spawn_controls(commands: &mut Commands, asset_server: Res<AssetServer>, m
     return results;
 }
 
+fn get_color_material_handle(path: &str, asset_server: &Res<AssetServer>, control: &Control, materials: &mut ResMut<Assets<ColorMaterial>>) -> Handle<ColorMaterial> {
+    let mut color_material: ColorMaterial = asset_server.load(path).into();
+    let color = control.fields.get_by_name("color");
+    if !color.is_empty() {
+        let color = to_bevy_color(parse_color(color));
+        color_material.color = color;
+    }
+    let color_material_handle = materials.add(color_material);
+    
+    return color_material_handle;
+}
+
 fn to_bevy_color(color_u32: u32) -> Color {
     let color_bytes= color_u32.to_le_bytes();
     let r = color_bytes[0] as f32 / 255.0;
     let g = color_bytes[1] as f32 / 255.0;
     let b = color_bytes[2] as f32 / 255.0;
     let a = color_bytes[3] as f32 / 255.0;
-    println!("r {}, g {}, b {}, a {}", r, g, b, a);
+
     let color = Color::Rgba { red: r, green: g, blue: b, alpha: a};
 
     return color;
@@ -245,7 +333,7 @@ fn calculate_top_left_position(control: &Control, controls: &Controls, screen_si
             let center_position = control.fields.get_by_name("center_position");
 
             if center_position.is_empty() {
-                return Vec2::new(0.0, 0.0);
+                return Vec2::new(0.0 - control_size.x * 0.5, 0.0 + control_size.y * 0.5);
             } else {
                 let cp = parse_vec2(&center_position);
                 let tlp = Vec2::new(cp.x - control_size.x * 0.5, cp.y + control_size.y * 0.5);
@@ -380,7 +468,8 @@ fn instantiate_sprite_bundle(
     size: Vec2,
     center_position: Vec3,
     scale: Vec3,
-    texture_handle: Handle<ColorMaterial>
+    material_handle: Handle<ColorMaterial>,
+    is_visible: bool,
 ) -> SpriteBundle {
 
     let sprite = Sprite::new(size);
@@ -393,7 +482,8 @@ fn instantiate_sprite_bundle(
     let bundle = SpriteBundle {
         transform,
         sprite,
-        material: texture_handle.clone(),
+        visible: Visible { is_visible, is_transparent: true},
+        material: material_handle.clone(),
         ..Default::default()
     };
 
