@@ -1,4 +1,7 @@
 use bevy::{prelude::*, window::WindowMode};
+//use bevy_mod_debug_console::ConsoleDebugPlugin;
+
+mod plugin;
 
 fn main() {
     App::build()
@@ -12,6 +15,8 @@ fn main() {
             ..Default::default()
         })
         .add_plugins(DefaultPlugins)
+        //.add_plugin(ConsoleDebugPlugin)
+        .add_plugin(crate::plugin::ControlsPlugin {})
         .add_startup_system(setup.system())
         .run();
 }
@@ -22,4 +27,4 @@ fn setup(mut commands: Commands, asset_server: Res<AssetServer>, materials: ResM
 
     let controls = gerg_ui::instantiate_controls_from_file("screen1.ui");
     let _entities = gerg_ui::spawn_controls(&mut commands, asset_server, materials, controls, Vec2::new(1920.0, 1080.0));
-}
\ No newline at end of file
+}
